@@ -27,6 +27,7 @@ import (
 	"github.com/armon/go-metrics"
 	"github.com/hashicorp/go-msgpack/codec"
 	"github.com/hashicorp/logutils"
+	"github.com/hashicorp/serf/coordinate"
 	"github.com/hashicorp/serf/serf"
 	"io"
 	"log"
@@ -64,6 +65,7 @@ const (
 	respondCommand         = "respond"
 	authCommand            = "auth"
 	statsCommand           = "stats"
+	getCoordinateCommand   = "get-coordinate"
 )
 
 const (
@@ -105,6 +107,15 @@ type authRequest struct {
 	AuthKey string
 }
 
+type coordinateRequest struct {
+	Node string
+}
+
+type coordinateResponse struct {
+	Coord coordinate.Coordinate
+	Ok    bool
+}
+
 type eventRequest struct {
 	Name     string
 	Payload  []byte
@@ -376,11 +387,6 @@ func (i *AgentIPC) listen() {
 			&codec.MsgpackHandle{RawToString: true, WriteExt: true})
 		client.enc = codec.NewEncoder(client.writer,
 			&codec.MsgpackHandle{RawToString: true, WriteExt: true})
-		if err != nil {
-			i.logger.Printf("[ERR] agent.ipc: Failed to create decoder: %v", err)
-			conn.Close()
-			continue
-		}
 
 		// Register the client
 		i.Lock()
@@ -521,6 +527,9 @@ func (i *AgentIPC) handleRequest(client *IPCClient, reqHeader *requestHeader) er
 	case statsCommand:
 		return i.handleStats(client, seq)
 
+	case getCoordinateCommand:
+		return i.handleGetCoordinate(client, seq)
+
 	default:
 		respHeader := responseHeader{Seq: seq, Error: unsupportedCommand}
 		client.Send(&respHeader, nil)
@@ -1024,6 +1033,32 @@ func (i *AgentIPC) handleStats(client *IPCClient, seq uint64) error {
 	return client.Send(&header, resp)
 }
 
+// handleGetCoordinate is used to get the cached coordinate for a node.
+func (i *AgentIPC) handleGetCoordinate(client *IPCClient, seq uint64) error {
+	var req coordinateRequest
+	if err := client.dec.Decode(&req); err != nil {
+		return fmt.Errorf("decode failed: %v", err)
+	}
+
+	// Fetch the coordinate.
+	var result coordinate.Coordinate
+	coord, ok := i.agent.Serf().GetCachedCoordinate(req.Node)
+	if ok {
+		result = *coord
+	}
+
+	// Respond
+	header := responseHeader{
+		Seq:   seq,
+		Error: errToString(nil),
+	}
+	resp := coordinateResponse{
+		Coord: result,
+		Ok:    ok,
+	}
+	return client.Send(&header, &resp)
+}
+
 // Used to convert an error to a string representation
 func errToString(err error) string {
 	if err == nil {
